from __future__ import unicode_literals

from django.db import models


class Log(models.Model):
    ERROR = 1
    DEBUG = 2
    WARN = 3
    INFO = 4
    LOG_LEVELS = (
        (ERROR, 'ERROR'),
        (DEBUG, 'DEBUG'),
        (WARN, 'WARN'),
        (INFO, 'INFO'),
    )

    log_level = models.IntegerField(
        choices=LOG_LEVELS,
        default=4)
    request_url = models.CharField(
        max_length=2083,
        blank=True)
    request_method = models.CharField(
        max_length=10,
        blank=True)
    get_data = models.TextField(
        blank=True)
    request_body = models.TextField(
        blank=True)
    cookies = models.TextField(
        blank=True)
    meta = models.TextField(
        blank=True)
    exception_type = models.CharField(
        max_length=2083,
        blank=True)
    message = models.TextField(
        blank=True)
    stack_trace = models.TextField(
        blank=True)
    user_id = models.IntegerField(
        blank=True,
        null=True)
    user_name = models.CharField(
        max_length=255,
        blank=True)
    request_browser = models.CharField(
        max_length=255,
        blank=True)
    request_os = models.CharField(
        max_length=255,
        blank=True)
    request_device = models.CharField(
        max_length=255,
        blank=True)
    response_body = models.TextField(
        blank=True)
    response_status = models.CharField(
        max_length=255,
        blank=True)
    response_headers = models.TextField(
        blank=True)
    response_content_type = models.CharField(
        max_length=255,
        blank=True)
    is_mobile = models.BooleanField(
        default=False)
    is_tablet = models.BooleanField(
        default=False)
    is_touch_capable = models.BooleanField(
        default=False)
    is_pc = models.BooleanField(
        default=False)
    is_bot = models.BooleanField(
        default=False)
    created_on = models.DateTimeField(
        auto_now_add=True)

    class Meta:
        verbose_name = "Log"
        verbose_name_plural = "Logs"

    def __unicode__(self):
        return self.exception_type


class RequestLog(models.Model):
    # GET = 1
    # POST = 2
    # PUT = 3
    # DELETE = 4
    # OPTIONS = 5
    # PATCH = 6
    # HEAD = 7
    # METHODS = (
    #     (GET, 'GET'),
    #     (POST, 'POST'),
    #     (PUT, 'PUT'),
    #     (DELETE, 'DELETE'),
    #     (OPTIONS, 'OPTIONS'),
    #     (PATCH, 'PATCH'),
    #     (HEAD, 'HEAD'),
    # )

    method = models.CharField(
        max_length=10,
        blank=True)
    url = models.CharField(
        max_length=2043,
        blank=True)
    request_data = models.TextField(
        blank=True)
    request_headers = models.TextField(
        blank=True)
    response_text = models.TextField(
        blank=True)
    response_status = models.IntegerField(
        null=True)
    response_reason = models.CharField(
        max_length=255,
        blank=True)
    response_time = models.IntegerField(
        blank=True)
    user_id = models.IntegerField(
        blank=True,
        null=True)
    user_name = models.CharField(
        max_length=255,
        blank=True)
    created_on = models.DateTimeField(
        auto_now_add=True)
    message = models.TextField(
        blank=True)

    class Meta:
        verbose_name = "Request Log"
        verbose_name_plural = "Request Logs"

    def __unicode__(self):
        return '%s %s' % (self.method, self.url)



from django.views.generic import View
from django.core.paginator import Paginator, EmptyPage
from django.shortcuts import render

from django.http import HttpResponseRedirect
from django.core.urlresolvers import reverse

from .models import (
    Log,
    RequestLog,
)
from .utils import Logger
from .response import render as logger_render
from .utils import RequestLogger


class AllLogs(View):
    template_name = 'logger/all_logs.html'

    def get(self, request):
        page = request.GET.get('page', 1)

        logs = Log.objects.all().order_by('-created_on')
        paginator = Paginator(logs, 10)

        try:
            logs = paginator.page(page)
        except EmptyPage:
            logs = paginator.page(paginator.num_pages)

        context = {'logs': logs}
        return render(request, self.template_name, context)

    def post(self, request):
        Log.objects.all().delete()
        return HttpResponseRedirect(reverse('logger:all_logs'))


class AllRequestLogs(View):
    template_name = 'logger/all_request_logs.html'

    def get(self, request):
        page = request.GET.get('page', 1)

        logs = RequestLog.objects.all().order_by('-created_on')
        paginator = Paginator(logs, 10)

        try:
            logs = paginator.page(page)
        except EmptyPage:
            logs = paginator.page(paginator.num_pages)

        context = {'logs': logs}
        return render(request, self.template_name, context)

    def post(self, request):
        RequestLog.objects.all().delete()
        return HttpResponseRedirect(reverse('logger:all_request_logs'))


class TestLogs(View):
    template_name = 'logger/logger_test.html'

    def get(self, request):
        logs = [
            Logger.log_info(request, 'Some info message. For Django render.'),
            Logger.log_debug(request, 'Some debug message. For Django render.'),
            Logger.log_warn(request, 'Some warn message. For Django render.'),
        ]

        context = {'some': 'data'}
        return logger_render(request, self.template_name, context, logs=logs)


class TestRequestLogs(View):
    template_name = 'logger/logger_test.html'

    def get(self, request):
        response = RequestLogger.get(
            'https://jsonplaceholder.typicode.com/posts/1',
            params={'query': 'value'},
            user=request.user,
            message='Some post request message')
        return render(request, self.template_name, {'text': response.text})



import traceback
import json

import requests

from .models import (
    Log,
    RequestLog,
)


class NoRequestException(Exception):
    pass


class NoExceptionException(Exception):
    pass


class ObjectLogger(object):
    def log_request(self, log=None, request=None, request_body=None):
        # -- General info
        log.request_url = request.get_full_path()
        log.request_method = request.method
        log.get_data = json.dumps(request.GET)
        log.request_body = request_body
        log.cookies = json.dumps(request.COOKIES)

        # --- Request meta info
        log.meta = ','.join('"%s": "%s"' % (k, str(v)) for k, v in request.META.items())
        log.meta = '{%s}' % log.meta
        log.meta = log.meta.replace('\\', '|')

        # --- User info
        if request.user and request.user.is_authenticated():
            log.user_id = request.user.id
            log.user_name = request.user.email

        # --- User agent info
        user_agent = request.user_agent
        # Browser
        log.request_browser = user_agent.browser
        # OS
        log.request_os = user_agent.os
        # Device
        log.request_device = user_agent.device
        # Device type
        log.is_mobile = user_agent.is_mobile
        log.is_tablet = user_agent.is_tablet
        log.is_touch_capable = user_agent.is_touch_capable
        log.is_pc = user_agent.is_pc
        log.is_bot = user_agent.is_bot

        return log

    def log_response(self, log, data, status=None, template_name=None, headers=None, content_type=None):
        log.response_body = json.dumps(data)
        log.response_status = status if status else 'None'
        log.response_headers = json.dumps(headers)
        log.response_content_type = content_type if content_type else 'None'

        return log

    def log_exception(self, log=None, exception=None):
        # --- Exception info
        log.exception_type = type(exception).__name__
        log.message = exception.message
        log.stack_trace = traceback.format_exc()

        return log


class Logger(object):
    @staticmethod
    def log_error(request=None, request_body=None, exception=None):
        if request and exception:
            log = Log(
                log_level=Log.ERROR)
            obj_logger = ObjectLogger()
            log = obj_logger.log_exception(log, exception)
            log = obj_logger.log_request(log, request, request_body)

            # --- Save
            log.save()
        elif request is None:
            raise NoRequestException('No http request found')
        elif exception is None:
            raise NoExceptionException('No exception found')

        return log

    @staticmethod
    def log_debug(request=None, message=None):
        stack_trace = ''.join(line for line in traceback.format_stack())
        message = message if message else ""
        if request:
            # -- General info
            log = Log(
                log_level=Log.DEBUG,
                message=message,
                stack_trace=stack_trace)
            obj_logger = ObjectLogger()
            log = obj_logger.log_request(log, request, request.body)

            # --- Save
            log.save()
            return log
        else:
            raise NoRequestException('No http request found')

    @staticmethod
    def log_warn(request=None, message=None):
        stack_trace = ''.join(line for line in traceback.format_stack())
        message = message if message else ""
        if request:
            # -- General info
            log = Log(
                log_level=Log.WARN,
                message=message,
                stack_trace=stack_trace)
            obj_logger = ObjectLogger()
            log = obj_logger.log_request(log, request, request.body)

            # --- Save
            log.save()
            return log
        else:
            raise NoRequestException('No http request found')

    @staticmethod
    def log_info(request=None, message=None):
        stack_trace = ''.join(line for line in traceback.format_stack())
        message = message if message else ""
        if request:
            # -- General info
            log = Log(
                log_level=Log.INFO,
                message=message,
                stack_trace=stack_trace)
            obj_logger = ObjectLogger()
            log = obj_logger.log_request(log, request, request.body)

            # --- Save
            log.save()
            return log
        else:
            raise NoRequestException('No http request found')

    @staticmethod
    def non_request_log(log_level=None, message=None):
        stack_trace = ''.join(line for line in traceback.format_stack())
        message = message if message else ""
        # -- General info
        log = Log(
            log_level=log_level if log_level else Log.INFO,
            message=message,
            stack_trace=stack_trace)

        # --- Save
        log.save()
        return log


class RequestObjectLogger(object):
    def log_request(self, log, request, data):
        # --- Request data
        log.method = request.method
        log.url = request.url
        log.request_data = data if isinstance(data, str) else json.dumps(data)
        headers = {val[0]: val[1] for val in request.headers.items()}
        log.request_headers = json.dumps(headers)

        return log

    def log_response(self, log, response, user, message):
        # --- Response data
        log.response_text = response.text
        log.response_status = response.status_code
        log.response_reason = response.reason
        log.response_time = response.elapsed.microseconds / 1000

        # --- User data
        if user:
            if user.is_authenticated():
                log.user_id = user.id
                log.user_name = user.email
        log.message = message if message else ''

        return log


class RequestLogger(object):
    @staticmethod
    def get(url, params=None, user=None, message=None, **kwargs):
        response = requests.get(url, params=params, **kwargs)

        log = RequestLog()

        obj_logger = RequestObjectLogger()

        log = obj_logger.log_request(log, response.request, params)
        log = obj_logger.log_response(log, response, user, message)

        # --- Save
        log.save()

        return response

    @staticmethod
    def post(url, data=None, json=None, user=None, message=None, **kwargs):
        response = requests.post(url, data=data, json=None, **kwargs)
        log = RequestLog()

        obj_logger = RequestObjectLogger()

        log = obj_logger.log_request(log, response.request, data)
        log = obj_logger.log_response(log, response, user, message)

        # --- Save
        log.save()

        return response

    @staticmethod
    def put(url, data=None, json=None, user=None, message=None, **kwargs):
        response = requests.put(url, data=data, json=None, **kwargs)
        log = RequestLog()

        obj_logger = RequestObjectLogger()

        log = obj_logger.log_request(log, response.request, data)
        log = obj_logger.log_response(log, response, user, message)

        # --- Save
        log.save()

        return response

    @staticmethod
    def delete(url, user=None, message=None, **kwargs):
        response = requests.delete(url, **kwargs)
        log = RequestLog()

        obj_logger = RequestObjectLogger()

        log = obj_logger.log_request(log, response.request, None)
        log = obj_logger.log_response(log, response, user, message)

        # --- Save
        log.save()

        return response

    @staticmethod
    def patch(url, data=None, json=None, user=None, message=None, **kwargs):
        response = requests.patch(url, data=data, json=None, **kwargs)
        log = RequestLog()

        obj_logger = RequestObjectLogger()

        log = obj_logger.log_request(log, response.request, data)
        log = obj_logger.log_response(log, response, user, message)

        # --- Save
        log.save()

        return response

    @staticmethod
    def head(url, user=None, message=None, **kwargs):
        response = requests.head(url, **kwargs)
        log = RequestLog()

        obj_logger = RequestObjectLogger()

        log = obj_logger.log_request(log, response.request, None)
        log = obj_logger.log_response(log, response, user, message)

        # --- Save
        log.save()

        return response



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Log',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('log_level', models.IntegerField(default=4, choices=[(1, 'ERROR'), (2, 'DEBUG'), (3, 'WARN'), (4, 'INFO')])),
                ('request_url', models.CharField(max_length=2083, blank=True)),
                ('request_method', models.CharField(max_length=10, blank=True)),
                ('get_data', models.TextField(blank=True)),
                ('request_body', models.TextField(blank=True)),
                ('cookies', models.TextField(blank=True)),
                ('meta', models.TextField(blank=True)),
                ('exception_type', models.CharField(max_length=2083, blank=True)),
                ('message', models.TextField(blank=True)),
                ('stack_trace', models.TextField(blank=True)),
                ('user_id', models.IntegerField(null=True, blank=True)),
                ('user_name', models.CharField(max_length=255, blank=True)),
                ('request_browser', models.CharField(max_length=255, blank=True)),
                ('request_os', models.CharField(max_length=255, blank=True)),
                ('request_device', models.CharField(max_length=255, blank=True)),
                ('response_body', models.TextField(blank=True)),
                ('response_status', models.CharField(max_length=255, blank=True)),
                ('response_headers', models.TextField(blank=True)),
                ('response_content_type', models.CharField(max_length=255, blank=True)),
                ('is_mobile', models.BooleanField(default=False)),
                ('is_tablet', models.BooleanField(default=False)),
                ('is_touch_capable', models.BooleanField(default=False)),
                ('is_pc', models.BooleanField(default=False)),
                ('is_bot', models.BooleanField(default=False)),
                ('created_on', models.DateTimeField(auto_now_add=True)),
            ],
            options={
                'verbose_name': 'Log',
                'verbose_name_plural': 'Logs',
            },
        ),
        migrations.CreateModel(
            name='RequestLog',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('method', models.CharField(max_length=10, blank=True)),
                ('url', models.CharField(max_length=2043, blank=True)),
                ('request_data', models.TextField(blank=True)),
                ('request_headers', models.TextField(blank=True)),
                ('response_text', models.TextField(blank=True)),
                ('response_status', models.IntegerField(null=True)),
                ('response_reason', models.CharField(max_length=255, blank=True)),
                ('response_time', models.IntegerField(blank=True)),
                ('user_id', models.IntegerField(null=True, blank=True)),
                ('user_name', models.CharField(max_length=255, blank=True)),
                ('created_on', models.DateTimeField(auto_now_add=True)),
                ('message', models.TextField(blank=True)),
            ],
            options={
                'verbose_name': 'Request Log',
                'verbose_name_plural': 'Request Logs',
            },
        ),
    ]



import decimal
import json
from django.shortcuts import render as django_render

from rest_framework.response import Response as DrfResponse
from .utils import ObjectLogger


def decimal_default(obj):
    if isinstance(obj, decimal.Decimal):
        return float(obj)
    raise TypeError


class Response(DrfResponse):
    """
    Wrapper around Django Rest Framework Response class. Can take one or a list of logs and logs the response.
    No overhead if no logs are passed.
    """
    def __init__(self, data, status=None, template_name=None, headers=None, content_type=None, logs=None):
        if logs:
            obj_logger = ObjectLogger()
            if not isinstance(logs, list):
                logs = [logs, ]
            for log in logs:
                log = obj_logger.log_response(
                    log,
                    data if isinstance(data, str) else json.dumps(data, default=decimal_default),
                    status=str(status),
                    headers=headers,
                    content_type=str(content_type))
                log.save()
        super(Response, self).__init__(
            data,
            status=status,
            template_name=template_name,
            headers=headers,
            content_type=content_type)


def render(request, template_name, context=None, content_type=None, status=None, using=None, logs=None):
    """
    Wrapper around Django render method. Can take one or a list of logs and logs the response.
    No overhead if no logs are passed.
    """
    if logs:
        obj_logger = ObjectLogger()
        if not isinstance(logs, list):
            logs = [logs, ]
        for log in logs:
            log = obj_logger.log_response(
                log,
                context,
                status=str(status),
                headers='',
                content_type=str(content_type))
            log.save()
    return django_render(
        request,
        template_name,
        context=context,
        content_type=content_type,
        status=status,
        using=using)



import traceback
import json

from logger.utils import Logger


class ErrorMiddleware(object):
    _initial_http_body = None

    def process_request(self, request):
        # this requires because for some reasons there is no way to access request.body
        # in the 'process_response' method.
        self._initial_http_body = request.body

    def process_exception(self, request, exception):
        Logger.log_error(request, self._initial_http_body, exception)

        return None


# def log_session_error(request, exception):
#     # -- General info
#     log = Log(
#         exception_type=type(exception).__name__,
#         message=exception.detail,
#         stack_trace=traceback.format_exc())
#     log.request_url = request.get_full_path()
#     log.request_method = request.method
#     log.get_data = json.dumps(request.GET)
#     log.post_data = json.dumps(request.POST)
#     log.request_body = '{}'
#     log.cookies = json.dumps(request.COOKIES)

#     # --- Request meta info
#     log.meta = ','.join('"%s": "%s"' % (k, str(v)) for k, v in request.META.items())
#     log.meta = '{%s}' % log.meta
#     log.meta = log.meta.replace('\\', '|')

#     # --- User info
#     if request.user.is_authenticated():
#         log.user_id = request.user.id
#         log.user_name = request.user.email

#     # --- User agent info
#     user_agent = request.user_agent
#     # Browser
#     log.request_browser = user_agent.browser
#     # OS
#     log.request_os = user_agent.os
#     # Device
#     log.request_device = user_agent.device
#     # Device type
#     log.is_mobile = user_agent.is_mobile
#     log.is_tablet = user_agent.is_tablet
#     log.is_touch_capable = user_agent.is_touch_capable
#     log.is_pc = user_agent.is_pc
#     log.is_bot = user_agent.is_bot

#     # --- Save
#     log.save()
#     return None



from django.contrib import admin

from .models import (
    Log,
    RequestLog,
)


class LogAdmin(admin.ModelAdmin):
    readonly_fields = [
        'log_level', 'request_url', 'request_method', 'get_data',
        'request_body', 'cookies', 'meta',
        'exception_type', 'message', 'stack_trace', 'user_id',
        'user_name', 'request_browser', 'request_os', 'request_device',
        'response_body', 'response_status', 'response_headers', 'response_content_type',
        'is_mobile', 'is_tablet', 'is_touch_capable', 'is_pc',
        'is_bot', 'created_on']

    def has_add_permission(self, request):
        return False


class RequestLogAdmin(admin.ModelAdmin):
    readonly_fields = [
        'method', 'url', 'request_data', 'request_headers',
        'response_text', 'response_status', 'response_reason',
        'response_time', 'created_on']

    def has_add_permission(self, request):
        return False


admin.site.register(Log, LogAdmin)
admin.site.register(RequestLog, RequestLogAdmin)



from django.conf.urls import url

from .views import (
    AllLogs,
    AllRequestLogs,
    TestLogs,
    TestRequestLogs,
)

urlpatterns = [
    url(r'^all/$', AllLogs.as_view(), name='all_logs'),
    url(r'^request/all/$', AllRequestLogs.as_view(), name='all_request_logs'),
    url(r'^test/$', TestLogs.as_view(), name='test'),
    url(r'^request/test/$', TestRequestLogs.as_view(), name='request_test'),
]



from distutils.core import setup


setup(
    name='simple_django_logger',
    packages=[
        'simple_django_logger',  # this must be the same as the name above
        'simple_django_logger.middleware',
        'simple_django_logger.migrations'],
    version='0.0.3',
    description='A basic logger for Django',
    author='Eshan Das',
    author_email='eshandasnit@gmail.com',
    url='https://github.com/eshandas/simple_django_logger',  # use the URL to the github repo
    download_url='https://github.com/eshandas/simple_django_logger/archive/0.0.3.tar.gz',  # Create a tag in github
    keywords=['django', 'logger'],
    classifiers=[],
)
